<?php
// $Id: ad_views.views.inc,v 1.13 2009/07/20 14:28:16 milesgillham Exp $
/*
 * @file
 *
 * This file provides multiple module Views interfaces:
 *    Ad Module (ad)
 *      Implemented as a separate base type.
 *      {ads}
 *      {ad_statistics}
 *      {ad_clicks}
 *      Ad External (ad_external)
 *        {ad_external}
 *      Ad HTML (ad_html)
 *        {ad_html}
 *      Ad Image (ad_image)
 *        {ad_image}
 *        {ad_image_format}
 *      Ad Notify (ad_notify)
 *        {ad_notify}
 *        {ad_owners}
 *        {ad_permissions}
 *        {ad_hosts}
 *      Ad Text (ad_text)
 *        {ad_text}
 *      Ad GeoIP (ad_geoip)
 *        {ad_geoip_codes}
 *        {ad_geoip_ads_country}
 *        {ad_geoip_ads_city}
 *        {ad_geoip_ads_region}
 *      Ad Memcache (ad_memcache)
 *        None.
 *
 */

/*
 * Implements hook_views_data()
 */

function ad_views_views_data() {
  $ad_data = array();
  // If the Ad module is present then provide some view support
  if (module_exists('ad')) {
    // Primary table(s)
    $ad_data = array_merge($ad_data, get_ad_views_data());
    // ad_image module
    $ad_data = array_merge($ad_data, get_ad_image_views_data());
    // ad_text module
    $ad_data = array_merge($ad_data, get_ad_text_views_data());
    // ad_external module
    $ad_data = array_merge($ad_data, get_ad_external_views_data());
  }
  return $ad_data;
}

#####################################################################
### Internal functions
#####################################################################

/*
 * Ad Module
 * ads is a valid base as it has first class data
 */
function get_ad_views_data() {
  $data = array();
  $data['ads']['table']['group'] = t('Ad');
  $data['ad_statistics']['table']['group'] = t('Ad Stats');
  $data['ad_clicks']['table']['group'] = t('Ad Clicks');
  // Base table is 'ads'
  $data['ads']['table']['base'] = array(
    'field' => 'aid',
    'title' => t('Advertisement'),
    'help' => t('The Ad item type allows views of Ad Module advertisements.'),
    'weight' => -10
  );
/*
  // TODO These probably aren't required.
  // Join to user
  $data['ads']['table']['join'] = array(
    'users' => array(
      'left_field' => 'uid',
      'field' => 'uid'
    )
  );
  $data['ad_clicks']['table']['join'] = array(
    'users' => array(
      'left_field' => 'uid',
      'field' => 'uid'
    )
  );
*/
  // ad_statistics is a related table to ads
  $data['ad_statistics']['table']['join']['ads'] = array(
    'left_field' => 'aid',
    'field' => 'aid'
  );
  // ad_clicks is a related table to ads
  $data['ad_clicks']['table']['join']['ads'] = array(
    'left_field' => 'aid',
    'field' => 'aid'
  );
  // Ad ID - 'aid'
  $data['ads']['aid'] = array(
    'title' => t('Aid'),
    // The help that appears on the UI
    'help' => t('The Ad ID of the ad.'),
    // Information for displaying
    'field' => array(
      'handler' => 'views_handler_field_numeric',
      'click sortable' => TRUE
    ),
    // Information for accepting as an argument
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
      'name field' => 'title',
      'numeric' => TRUE,
      'validate type' => 'aid'
    ),
    // Information for accepting as a filter
    'filter' => array(
      'handler' => 'views_handler_filter_numeric'
    ),
    // Information for sorting
    'sort' => array(
      'handler' => 'views_handler_sort'
    )
  );
  // Ad uid - 'uid'
  $data['ads']['uid'] = array(
    'title' => t('User'),
    // The help that appears on the UI
    'help' => t('The {users}.uid that owns this node; initially, this is the user that created it.'),
    // Table relationship
    'relationship' => array(
      'label' => t('User'),
      'base' => 'users',
      'base field' => 'uid',
      // This allows us to not show this relationship if the base is already
      // user so users won't create circular relationships.
      'skip base' => array('users')
    )
  );
  // Ad Status - 'adstatus'
  $data['ads']['adstatus'] = array(
    'title' => t('Status'),
    // The help that appears on the UI
    'help' => t('The approval status of the ad.'),
    // Information for displaying
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE
    ),
    // Information for accepting as an argument
    'argument' => array(
      'handler' => 'views_handler_argument_string',
      'name field' => 'title',
      'numeric' => FALSE,
    ),
    // Information for accepting as a filter
    'filter' => array(
      'handler' => 'views_handler_filter_string'
    ),
    // Information for sorting
    'sort' => array(
      'handler' => 'views_handler_sort'
    )
  );
  // Ad Type - 'adtype'
  $data['ads']['adtype'] = array(
    'title' => t('Type'),
    // The help that appears on the UI
    'help' => t('The type of ad.'),
    // Information for displaying
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE
    ),
    // Information for accepting as an argument
    'argument' => array(
      'handler' => 'views_handler_argument_string',
      'name field' => 'title',
      'numeric' => FALSE,
    ),
    // Information for accepting as a filter
    'filter' => array(
      'handler' => 'views_handler_filter_string'
    ),
    // Information for sorting
    'sort' => array(
      'handler' => 'views_handler_sort'
    )
  );
  // Ad Redirect - 'redirect'
  $data['ads']['redirect'] = array(
    'title' => t('Redirect URL'),
    // The help that appears on the UI
    'help' => t('The URL to redirect to.'),
    // Information for displaying
    'field' => array(
      'handler' => 'views_handler_field_url',
      'click sortable' => TRUE
    ),
    // Information for accepting as an argument
    'argument' => array(
      'handler' => 'views_handler_argument_string',
      'name field' => 'title',
      'numeric' => FALSE,
    ),
    // Information for accepting as a filter
    'filter' => array(
      'handler' => 'views_handler_filter_string'
    ),
    // Information for sorting
    'sort' => array(
      'handler' => 'views_handler_sort'
    )
  );
  // Ad autoactivate - 'autoactivate'
  $data['ads']['autoactivate'] = array(
    'title' => t('Auto activate'),
    // The help that appears on the UI
    'help' => t('Is ad autoactivating?'),
    // Information for displaying
    'field' => array(
      'handler' => 'views_handler_field_boolean',
      'click sortable' => TRUE
    ),
    // Information for accepting as an argument
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
      'name field' => 'title',
      'numeric' => TRUE,
    ),
    // Information for accepting as a filter
    'filter' => array(
      'handler' => 'views_handler_filter_boolean_operator'
    ),
    // Information for sorting
    'sort' => array(
      'handler' => 'views_handler_sort'
    )
  );
  // Ad autoactivated - 'autoactivated'
  $data['ads']['autoactivated'] = array(
    'title' => t('Auto activated'),
    // The help that appears on the UI
    'help' => t('Is ad autoactivated?'),
    // Information for displaying
    'field' => array(
      'handler' => 'views_handler_field_boolean',
      'click sortable' => TRUE
    ),
    // Information for accepting as an argument
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
      'name field' => 'title',
      'numeric' => TRUE,
    ),
    // Information for accepting as a filter
    'filter' => array(
      'handler' => 'views_handler_filter_boolean_operator'
    ),
    // Information for sorting
    'sort' => array(
      'handler' => 'views_handler_sort'
    )
  );
  // Ad autoexpire - 'autoexpire'
  $data['ads']['autoexpire'] = array(
    'title' => t('Auto expire'),
    // The help that appears on the UI
    'help' => t('Is ad autoexpiring?'),
    // Information for displaying
    'field' => array(
      'handler' => 'views_handler_field_boolean',
      'click sortable' => TRUE
    ),
    // Information for accepting as an argument
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
      'name field' => 'title',
      'numeric' => TRUE,
    ),
    // Information for accepting as a filter
    'filter' => array(
      'handler' => 'views_handler_filter_boolean_operator'
    ),
    // Information for sorting
    'sort' => array(
      'handler' => 'views_handler_sort'
    )
  );
  // Ad autoexpired - 'autoexpired'
  $data['ads']['autoexpired'] = array(
    'title' => t('Auto expired'),
    // The help that appears on the UI
    'help' => t('Is ad autoexpired?'),
    // Information for displaying
    'field' => array(
      'handler' => 'views_handler_field_boolean',
      'click sortable' => TRUE
    ),
    // Information for accepting as an argument
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
      'name field' => 'title',
      'numeric' => TRUE,
    ),
    // Information for accepting as a filter
    'filter' => array(
      'handler' => 'views_handler_filter_boolean_operator'
    ),
    // Information for sorting
    'sort' => array(
      'handler' => 'views_handler_sort'
    )
  );
  // Ad activated - 'activated'
  $data['ads']['activated'] = array(
    'title' => t('Activated'),
    // The help that appears on the UI
    'help' => t('Is ad activated?'),
    // Information for displaying
    'field' => array(
      'handler' => 'views_handler_field_boolean',
      'click sortable' => TRUE
    ),
    // Information for accepting as an argument
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
      'name field' => 'title',
      'numeric' => TRUE,
    ),
    // Information for accepting as a filter
    'filter' => array(
      'handler' => 'views_handler_filter_boolean_operator'
    ),
    // Information for sorting
    'sort' => array(
      'handler' => 'views_handler_sort'
    )
  );
  // Ad maxviews - 'maxviews'
  $data['ads']['maxviews'] = array(
    'title' => t('Max views'),
    // The help that appears on the UI
    'help' => t('Maximum ad impressions.'),
    // Information for displaying
    'field' => array(
      'handler' => 'views_handler_field_numeric',
      'click sortable' => TRUE
    ),
    // Information for accepting as an argument
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
      'name field' => 'title',
      'numeric' => TRUE,
    ),
    // Information for accepting as a filter
    'filter' => array(
      'handler' => 'views_handler_filter_numeric'
    ),
    // Information for sorting
    'sort' => array(
      'handler' => 'views_handler_sort'
    )
  );
  // Ad maxclicks - 'maxclicks'
  $data['ads']['maxclicks'] = array(
    'title' => t('Max clicks'),
    // The help that appears on the UI
    'help' => t('Maximum ad clicks.'),
    // Information for displaying
    'field' => array(
      'handler' => 'views_handler_field_numeric',
      'click sortable' => TRUE
    ),
    // Information for accepting as an argument
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
      'name field' => 'title',
      'numeric' => TRUE,
    ),
    // Information for accepting as a filter
    'filter' => array(
      'handler' => 'views_handler_filter_numeric'
    ),
    // Information for sorting
    'sort' => array(
      'handler' => 'views_handler_sort'
    )
  );
  // Ad expired - 'expired'
  $data['ads']['expired'] = array(
    'title' => t('Expired'),
    // The help that appears on the UI
    'help' => t('Is ad expired?'),
    // Information for displaying
    'field' => array(
      'handler' => 'views_handler_field_boolean',
      'click sortable' => TRUE
    ),
    // Information for accepting as an argument
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
      'name field' => 'title',
      'numeric' => TRUE,
    ),
    // Information for accepting as a filter
    'filter' => array(
      'handler' => 'views_handler_filter_boolean_operator'
    ),
    // Information for sorting
    'sort' => array(
      'handler' => 'views_handler_sort'
    )
  );
  // Ad statistics - 'date'
  $data['ad_statistics']['date'] = array(
    'title' => t('Action date'),
    // The help that appears on the UI
    'help' => t('Date when action was made.'),
    // Information for displaying
    'field' => array(
      'handler' => 'views_handler_field_date',
      'click sortable' => TRUE
    ),
    // Information for accepting as an argument
    'argument' => array(
      'handler' => 'views_handler_argument_date',
      'name field' => 'title',
      'numeric' => TRUE,
    ),
    // Information for accepting as a filter
    'filter' => array(
      'handler' => 'views_handler_filter_date'
    ),
    // Information for sorting
    'sort' => array(
      'handler' => 'views_handler_sort_date'
    )
  );
  // Ad statistics - 'action'
  // TODO - can we make this an enum with some custom
  // handlers?
  $data['ad_statistics']['action'] = array(
    'title' => t('Action type'),
    // The help that appears on the UI
    'help' => t('Actions: "view", "click", "enable", "disable".'),
    // Information for displaying
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE
    ),
    // Information for accepting as an argument
    'argument' => array(
      'handler' => 'views_handler_argument_string',
      'name field' => 'title',
      'numeric' => FALSE,
    ),
    // Information for accepting as a filter
    'filter' => array(
      'handler' => 'views_handler_filter_string'
    ),
    // Information for sorting
    'sort' => array(
      'handler' => 'views_handler_sort'
    )
  );
  // Ad statistics - 'adgroup'
  $data['ad_statistics']['adgroup'] = array(
    'title' => t('Ad group'),
    // The help that appears on the UI
    'help' => t('Ad group.'),
    // Information for displaying
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE
    ),
    // Information for accepting as an argument
    'argument' => array(
      'handler' => 'views_handler_argument_string',
      'name field' => 'title',
      'numeric' => FALSE,
    ),
    // Information for accepting as a filter
    'filter' => array(
      'handler' => 'views_handler_filter_string'
    ),
    // Information for sorting
    'sort' => array(
      'handler' => 'views_handler_sort'
    )
  );
  // Ad statistics - 'hostid'
  $data['ad_statistics']['hostid'] = array(
    'title' => t('Hostid'),
    // The help that appears on the UI
    'help' => t('Host from which action was made.'),
    // Information for displaying
    'field' => array(
      'handler' => 'views_handler_field_numeric',
      'click sortable' => TRUE
    ),
    // Information for accepting as an argument
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
      'name field' => 'title',
      'numeric' => TRUE,
      'validate type' => 'aid'
    ),
    // Information for accepting as a filter
    'filter' => array(
      'handler' => 'views_handler_filter_numeric'
    ),
    // Information for sorting
    'sort' => array(
      'handler' => 'views_handler_sort'
    )
  );
  // Ad statistics - 'count'
  $data['ad_statistics']['count'] = array(
    'title' => t('Action count'),
    // The help that appears on the UI
    'help' => t('Count of actions triggered.'),
    // Information for displaying
    'field' => array(
      'handler' => 'views_handler_field_numeric',
      'click sortable' => TRUE
    ),
    // Information for accepting as an argument
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
      'name field' => 'title',
      'numeric' => TRUE,
      'validate type' => 'aid'
    ),
    // Information for accepting as a filter
    'filter' => array(
      'handler' => 'views_handler_filter_numeric'
    ),
    // Information for sorting
    'sort' => array(
      'handler' => 'views_handler_sort'
    )
  );
  // Ad statistics - 'extra'
  $data['ad_statistics']['extra'] = array(
    'title' => t('Extra'),
    // The help that appears on the UI
    'help' => t('Allow add-on modules to provide additional statistics granularity.'),
    // Information for displaying
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE
    ),
    // Information for accepting as an argument
    'argument' => array(
      'handler' => 'views_handler_argument_string',
      'name field' => 'title',
      'numeric' => FALSE,
    ),
    // Information for accepting as a filter
    'filter' => array(
      'handler' => 'views_handler_filter_string'
    ),
    // Information for sorting
    'sort' => array(
      'handler' => 'views_handler_sort'
    )
  );
  // Ad clicks - 'uid'
  $data['ad_clicks']['uid'] = array(
    'title' => t('User'),
    // The help that appears on the UI
    'help' => t('The {users}.uid that clicked this node.'),
    // Table relationship
    'relationship' => array(
      'label' => t('User'),
      'base' => 'users',
      'base field' => 'uid',
      // This allows us to not show this relationship if the base is already
      // user so users won't create circular relationships.
      'skip base' => array('users')
    )
  );
  // Ad clicks - 'status'
  $data['ad_clicks']['status'] = array(
    'title' => t('Status'),
    // The help that appears on the UI
    'help' => t('Status.'),
    // Information for displaying
    'field' => array(
      'handler' => 'views_handler_field_numeric',
      'click sortable' => TRUE
    ),
    // Information for accepting as an argument
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
      'name field' => 'title',
      'numeric' => TRUE,
      'validate type' => 'aid'
    ),
    // Information for accepting as a filter
    'filter' => array(
      'handler' => 'views_handler_filter_numeric'
    ),
    // Information for sorting
    'sort' => array(
      'handler' => 'views_handler_sort'
    )
  );
  // Ad clicks - 'hostname'
  $data['ad_clicks']['hostname'] = array(
    'title' => t('Host name'),
    // The help that appears on the UI
    'help' => t('Host from which action was made.'),
    // Information for displaying
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE
    ),
    // Information for accepting as an argument
    'argument' => array(
      'handler' => 'views_handler_argument_string',
      'name field' => 'title',
      'numeric' => FALSE,
    ),
    // Information for accepting as a filter
    'filter' => array(
      'handler' => 'views_handler_filter_string'
    ),
    // Information for sorting
    'sort' => array(
      'handler' => 'views_handler_sort'
    )
  );
  // Ad clicks - 'user_agent'
  $data['ad_clicks']['user_agent'] = array(
    'title' => t('User agent'),
    // The help that appears on the UI
    'help' => t('Clicker\'s browser agent.'),
    // Information for displaying
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE
    ),
    // Information for accepting as an argument
    'argument' => array(
      'handler' => 'views_handler_argument_string',
      'name field' => 'title',
      'numeric' => FALSE,
    ),
    // Information for accepting as a filter
    'filter' => array(
      'handler' => 'views_handler_filter_string'
    ),
    // Information for sorting
    'sort' => array(
      'handler' => 'views_handler_sort'
    )
  );
  // Ad clicks - 'adgroup'
  $data['ad_clicks']['adgroup'] = array(
    'title' => t('Ad group'),
    // The help that appears on the UI
    'help' => t('Ad group.'),
    // Information for displaying
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE
    ),
    // Information for accepting as an argument
    'argument' => array(
      'handler' => 'views_handler_argument_string',
      'name field' => 'title',
      'numeric' => FALSE,
    ),
    // Information for accepting as a filter
    'filter' => array(
      'handler' => 'views_handler_filter_string'
    ),
    // Information for sorting
    'sort' => array(
      'handler' => 'views_handler_sort'
    )
  );
  // Ad clicks - 'hostid'
  $data['ad_clicks']['hostid'] = array(
    'title' => t('Hostid'),
    // The help that appears on the UI
    'help' => t('Host from which action was made.'),
    // Information for displaying
    'field' => array(
      'handler' => 'views_handler_field_numeric',
      'click sortable' => TRUE
    ),
    // Information for accepting as an argument
    'argument' => array(
      'handler' => 'views_handler_argument_numeric',
      'name field' => 'title',
      'numeric' => TRUE,
      'validate type' => 'aid'
    ),
    // Information for accepting as a filter
    'filter' => array(
      'handler' => 'views_handler_filter_numeric'
    ),
    // Information for sorting
    'sort' => array(
      'handler' => 'views_handler_sort'
    )
  );
  // Ad clicks - 'url'
  $data['ad_clicks']['url'] = array(
    'title' => t('URL'),
    // The help that appears on the UI
    'help' => t('Clicked URL.'),
    // Information for displaying
    'field' => array(
      'handler' => 'views_handler_field_url',
      'click sortable' => TRUE
    ),
    // Information for accepting as an argument
    'argument' => array(
      'handler' => 'views_handler_argument_string',
      'name field' => 'title',
      'numeric' => FALSE,
    ),
    // Information for accepting as a filter
    'filter' => array(
      'handler' => 'views_handler_filter_string'
    ),
    // Information for sorting
    'sort' => array(
      'handler' => 'views_handler_sort'
    )
  );
  // Ad clicks - 'timestamp'
  $data['ad_clicks']['timestamp'] = array(
    'title' => t('Time stamp'),
    // The help that appears on the UI
    'help' => t('Date when action was made.'),
    // Information for displaying
    'field' => array(
      'handler' => 'views_handler_field_date',
      'click sortable' => TRUE
    ),
    // Information for accepting as an argument
    'argument' => array(
      'handler' => 'views_handler_argument_date',
      'name field' => 'title',
      'numeric' => TRUE,
    ),
    // Information for accepting as a filter
    'filter' => array(
      'handler' => 'views_handler_filter_date'
    ),
    // Information for sorting
    'sort' => array(
      'handler' => 'views_handler_sort_date'
    )
  );
  // Ad clicks - 'extra'
  $data['ad_clicks']['extra'] = array(
    'title' => t('Extra'),
    // The help that appears on the UI
    'help' => t('Allow add-on modules to provide additional statistics granularity.'),
    // Information for displaying
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE
    ),
    // Information for accepting as an argument
    'argument' => array(
      'handler' => 'views_handler_argument_string',
      'name field' => 'title',
      'numeric' => FALSE,
    ),
    // Information for accepting as a filter
    'filter' => array(
      'handler' => 'views_handler_filter_string'
    ),
    // Information for sorting
    'sort' => array(
      'handler' => 'views_handler_sort'
    )
  );
  return $data;
}

function get_ad_text_views_data() {
  $name = 'ad_text';
  $data = array();
  if ((db_table_exists($name)) && ((module_exists($name)) || (variable_get('ad_views_always_show_'. $name, FALSE)))) {
    // All ads are in a single table group
    $data['ad_text']['table']['group'] = t('Ad Text');
    // ad_text is a related table to ads
    $data['ad_text']['table']['join']['ads'] = array(
      'left_field' => 'aid',
      'field' => 'aid'
    );
    // Ad URL - 'url'
    $data['ad_text']['url'] = array(
      'title' => t('URL'),
      // The help that appears on the UI
      'help' => t('The ad URL.'),
      // Information for displaying
      'field' => array(
        'handler' => 'views_handler_field_url',
        'click sortable' => TRUE
      ),
      // Information for accepting as an argument
      'argument' => array(
        'handler' => 'views_handler_argument_string',
        'name field' => 'title',
        'numeric' => FALSE,
      ),
      // Information for accepting as a filter
      'filter' => array(
        'handler' => 'views_handler_filter_string'
      ),
      // Information for sorting
      'sort' => array(
        'handler' => 'views_handler_sort'
      )
    );
  }
    // Ad AdHeader - 'adheader'
    $data['ad_text']['adheader'] = array(
      'title' => t('Ad Header'),
      // The help that appears on the UI
      'help' => t('The ad header.'),
      // Information for displaying
      'field' => array(
        'handler' => 'views_handler_field',
        'click sortable' => TRUE
      ),
      // Information for accepting as an argument
      'argument' => array(
        'handler' => 'views_handler_argument_string',
        'name field' => 'title',
        'numeric' => FALSE,
      ),
      // Information for accepting as a filter
      'filter' => array(
        'handler' => 'views_handler_filter_string'
      ),
      // Information for sorting
      'sort' => array(
        'handler' => 'views_handler_sort'
      )
    );
    // Ad AdBody - 'adbody'
    $data['ad_text']['adbody'] = array(
      'title' => t('Ad Body'),
      // The help that appears on the UI
      'help' => t('The ad body.'),
      // Information for displaying
      'field' => array(
        'handler' => 'views_handler_field',
        'click sortable' => TRUE
      ),
      // Information for accepting as an argument
      'argument' => array(
        'handler' => 'views_handler_argument_string',
        'name field' => 'title',
        'numeric' => FALSE,
      ),
      // Information for accepting as a filter
      'filter' => array(
        'handler' => 'views_handler_filter_string'
      ),
      // Information for sorting
      'sort' => array(
        'handler' => 'views_handler_sort'
      )
    );
  return $data;
}

function get_ad_image_views_data() {
  $name = 'ad_image';
  $data = array();
  if ((db_table_exists($name)) && ((module_exists($name)) || (variable_get('ad_views_always_show_'. $name, FALSE)))) {
    // All ads are in a single table group
    $data['ad_image']['table']['group'] = t('Ad Image');
    // ad_image is a related table to ads
    $data['ad_image']['table']['join']['ads'] = array(
      'left_field' => 'aid',
      'field' => 'aid'
    );

/*
    // Join to files
    // This probably isn't required
    $data['ad_image']['table']['join'] = array(
      'files' => array(
        'left_field' => 'fid',
        'field' => 'fid'
      )
    );
*/
    // Ad image - 'fid'
    $data['ad_image']['fid'] = array(
      'title' => t('Fid'),
      // The help that appears on the UI
      'help' => t('The {files}.fid File ID that this ad relates to.'),
      // Table relationship
      'relationship' => array(
        'label' => t('Files'),
        'base' => 'files',
        'base field' => 'fid',
        // This allows us to not show this relationship if the base is already
        // files so files won't create circular relationships.
        'skip base' => array('files')
      )
    );
    // Ad image - 'url'
    $data['ad_image']['url'] = array(
      'title' => t('URL'),
      // The help that appears on the UI
      'help' => t('The ad URL.'),
      // Information for displaying
      'field' => array(
        'handler' => 'views_handler_field_url',
        'click sortable' => TRUE
      ),
      // Information for accepting as an argument
      'argument' => array(
        'handler' => 'views_handler_argument_string',
        'name field' => 'title',
        'numeric' => FALSE,
      ),
      // Information for accepting as a filter
      'filter' => array(
        'handler' => 'views_handler_filter_string'
      ),
      // Information for sorting
      'sort' => array(
        'handler' => 'views_handler_sort'
      )
    );
    // Ad image - 'tooltip'
    $data['ad_image']['tooltip'] = array(
      'title' => t('Tooltip'),
      // The help that appears on the UI
      'help' => t('The tooltip.'),
      // Information for displaying
      'field' => array(
        'handler' => 'views_handler_field',
        'click sortable' => TRUE
      ),
      // Information for accepting as an argument
      'argument' => array(
        'handler' => 'views_handler_argument_string',
        'name field' => 'title',
        'numeric' => FALSE,
      ),
      // Information for accepting as a filter
      'filter' => array(
        'handler' => 'views_handler_filter_string'
      ),
      // Information for sorting
      'sort' => array(
        'handler' => 'views_handler_sort'
      )
    );
    // Ad image - 'remote_image'
    $data['ad_image']['remote_image'] = array(
      'title' => t('Remote image'),
      // The help that appears on the UI
      'help' => t('The remote image URL.'),
      // Information for displaying
      'field' => array(
        'handler' => 'views_handler_field_url',
        'click sortable' => TRUE
      ),
      // Information for accepting as an argument
      'argument' => array(
        'handler' => 'views_handler_argument_string',
        'name field' => 'title',
        'numeric' => FALSE,
      ),
      // Information for accepting as a filter
      'filter' => array(
        'handler' => 'views_handler_filter_string'
      ),
      // Information for sorting
      'sort' => array(
        'handler' => 'views_handler_sort'
      )
    );
    // Ad image - 'width'
    $data['ad_image']['width'] = array(
      'title' => t('Width'),
      // The help that appears on the UI
      'help' => t('Ad width.'),
      // Information for displaying
      'field' => array(
        'handler' => 'views_handler_field_numeric',
        'click sortable' => TRUE
      ),
      // Information for accepting as an argument
      'argument' => array(
        'handler' => 'views_handler_argument_numeric',
        'name field' => 'title',
        'numeric' => TRUE,
        'validate type' => 'aid'
      ),
      // Information for accepting as a filter
      'filter' => array(
        'handler' => 'views_handler_filter_numeric'
      ),
      // Information for sorting
      'sort' => array(
        'handler' => 'views_handler_sort'
      )
    );
    // Ad image - 'height'
    $data['ad_image']['height'] = array(
      'title' => t('Height'),
      // The help that appears on the UI
      'help' => t('Ad height.'),
      // Information for displaying
      'field' => array(
        'handler' => 'views_handler_field_numeric',
        'click sortable' => TRUE
      ),
      // Information for accepting as an argument
      'argument' => array(
        'handler' => 'views_handler_argument_numeric',
        'name field' => 'title',
        'numeric' => TRUE,
        'validate type' => 'aid'
      ),
      // Information for accepting as a filter
      'filter' => array(
        'handler' => 'views_handler_filter_numeric'
      ),
      // Information for sorting
      'sort' => array(
        'handler' => 'views_handler_sort'
      )
    );
  }
  if (db_table_exists('ad_image_format')) {
  }
  return $data;
}

function get_ad_external_views_data() {
  $name = 'ad_external';
  $data = array();
  if ((db_table_exists($name)) && ((module_exists($name)) || (variable_get('ad_views_always_show_'. $name, FALSE)))) {
    // TODO
  }
  return $data;
}

// TODO all other Ads modules
